unit Fossil;
{ (C) 1987 W. B. Fair, Jr. "All Rights Reserved"

  This Unit provides basic I/O services via a FOSSIL program.
  The FOSSIL is assumed to be previously installed and provides
  a degree of system independence to communications software.

  12/15/87 Initial coding

  This unit was modifyed by DDT for use within a BBS first the baud rate junk
  was changed so it got the right parity etc and then a bit was added to
  the Xmit char routine to check for carrier and to die if it is not there
  }

interface
  uses DOS;
  const
    MaxPort = 2;
  type
    ComPort = byte;
  var
    FossilInstalled,
    ignorecarrier   : boolean;
    FossilMaxFcn    : byte;
    FossilVersion   : byte;
    PortStat        : array[0..MaxPort] of word;

  procedure SystemReboot;
  procedure InitFossil    (port : ComPort);
  procedure UnInitFossil  (port : ComPort);
  procedure SetBaud       (port : ComPort; baud : integer);
  procedure XmitPort      (c : byte; port : ComPort);
  procedure XmitChar      (c : char; port : ComPort);
  function  RcvPort       (port : ComPort) : byte;
  function  RcvChar       (port : ComPort) : char;
  procedure GetPortStatus (port : ComPort);
  function  PortDataRdy   (port : ComPort) : boolean;
  procedure LowerDTR      (port : ComPort);
  procedure RaiseDTR      (port : ComPort);

implementation

  var
    Regs      : Registers;

  procedure SystemReboot;
  begin
    Regs.AH:= $17;
    Regs.AL:= 0;
    Intr($14, Regs);
  end; {SystemReboot }

  procedure InitFossil(port : ComPort);
  begin
    FossilInstalled:= false;
    Regs.DX:= port-1;
    Regs.BX:= 0;
    Regs.CX:= 0;
    Regs.AH:= 4;
    Intr( $14, Regs);
    FossilInstalled:= (Regs.AX = $1954);
    FossilMaxFcn:= Regs.BL;
    FossilVersion:= Regs.BH;
  end; { InitFossil }

  procedure SetBaud(port : ComPort; baud : integer);
  begin { assume 8,n,1 }
    with Regs do
    case baud of
       300: AL:= $43;
       600: AL:= $63;
      1200: AL:= $83;
      2400: AL:= $A3;
      4800: AL:= $C3;
      9600: AL:= $E3;
     19200: AL:= 3;
      else  AL:= $83;
    end; { case }
    Regs.DX:= port-1;
    Regs.AH:= 0;
    Intr($14, Regs);
  end; { SetBaud }

var f:text;
  procedure XmitPort(c : byte; port : ComPort);
  begin
    Regs.DX:= port-1;
    Regs.AL:= c;
    Regs.AH:= 1;
    Intr( $14, Regs);
    PortStat[Ord(port)]:= Regs.AX;
    write(f,chr(c));
    if (((portstat[port] and $80)=0) and (not ignorecarrier)) then
      begin
      writeln('Warning lost carrier...---...===+++kjsdhg');
      uninitfossil(port);
      halt(42);
      end;
  end; { Xmit }

  procedure XmitChar(c: char; port : Comport);
  begin
    XmitPort(ord(c),port);
  end; { Xmit char }

  function RcvPort(port : ComPort) : byte;
  begin
    Regs.DX:= port-1;
    Regs.AH:= 2;
    Intr($14, Regs);
    RcvPort:= Regs.AL;
  end; { RcvPort }

  function RcvChar(port : ComPort) : char;
  begin
    RcvChar := chr(RcvPort(port));
  end;

  procedure GetPortStatus(port : ComPort);
  begin
    Regs.DX:= port-1;
    Regs.AH:= 3;
    Intr($14, Regs);
    PortStat[Ord(port)]:= Regs.AX;
  end; { GetPortStatus }

  function PortDataRdy(port : ComPort) : boolean;
  begin
    GetPortStatus(port);
    PortDataRdy:= ((PortStat[Ord(port)] and $100) <> 0);
  end; { PortDataRdy }

  procedure UnInitFossil(port : ComPort);
  begin
    Regs.DX:= port-1;
    Regs.AH:= 5;
    Intr($14, Regs);
    FossilInstalled:= false;
  end; { UnInitFossil }

  procedure RaiseDTR(port : ComPort);
  begin
    Regs.DX:= port-1;
    Regs.AL:= 1;
    Regs.AH:= 6;
    Intr($14, Regs);
  end; { RaiseDTR }

  procedure LowerDTR(port : ComPort);
  begin
    Regs.DX:= port-1;
    Regs.AL:= 0;
    Regs.AH:= 6;
    Intr($14, Regs);
  end; { LowerDTR }

begin
  assign(f,'junk.ans');
  rewrite(f);
  { Initialization Code }
  FossilInstalled:= false;
  ignorecarrier :=false;
end.